/**************************************************************************
 * HeXHub Winamp Plugin Copyright (C) 2007-2008 Mikhail Polenin           *
 * HeXHub is Copyright(C) Albu Cristian Laurentiu                         *
 * This plugin is licensed under the Open Software License 3.0            *
 * Please read the license before making a modification to this plugin.   *
 * Failure to follow the terms of the license will have severe            *
 * consequences                                                           *
 **************************************************************************/

#include "listclass.h"

extern char dir[MAX_PATH];

chatlist::chatlist()
{
	this->first = 0;
	this->last = 0;
	InitializeCriticalSection(&cs);
}

BOOL chatlist::Clear()
{
	listitem *pos = this->first;
	listitem *forward;
	EnterCriticalSection(&cs);
	while(pos != 0)
	{
		forward = pos->next;
		delete pos;
		pos = forward;
	}
	this->first = 0;
	this->last = 0;
	LeaveCriticalSection(&cs);
	return true;
}

chatlist::~chatlist()
{
	listitem *pos = this->first;
	listitem *forward;
	EnterCriticalSection(&cs);
	while(pos != 0)
	{
		forward = pos->next;
		delete pos;
		pos = forward;
	}
	LeaveCriticalSection(&cs);
	DeleteCriticalSection(&cs);
}

BOOL chatlist::Add(char *str)
{
	listitem *it = new listitem;
	memcpywithnull(it->str, str, 40);
	EnterCriticalSection(&cs);
	if(this->first == 0){
		this->first = it;
		this->last = it;
	} else {
		this->last->next = it;
		this->last = it;
	}
	LeaveCriticalSection(&cs);
	return true;
}

BOOL chatlist::Find(char *str)
{
	listitem *it = this->first;
	listitem *forward;
	EnterCriticalSection(&cs);
	while(it != 0)
	{
		forward = it->next;
		if(strcmp(_strlwr(str), _strlwr(it->str)) == 0)
		{
			LeaveCriticalSection(&cs);
			return true;
		}
		it = forward;
	}
	LeaveCriticalSection(&cs);
	return false;
}

listitem *chatlist::Previous(listitem *idx)
{
	EnterCriticalSection(&cs);
	listitem *bla = this->first;
	if(idx = this->first)
		return this->first;
	while(bla->next != idx) bla = bla->next;
	LeaveCriticalSection(&cs);
	return bla;
}

BOOL chatlist::Remove(char *str)
{
	listitem *temp;
	listitem *it = this->first;
	listitem *forward;
	EnterCriticalSection(&cs);
	while(it != 0)
	{
		forward = it->next;
		if(strcmp(_strlwr(str), _strlwr(it->str)) == 0)
		{
			if(it == this->first) { 
				temp = this->first; 
				this->first = this->first->next; 
				delete temp; 
			}
			else if(it == this->last) { 
				temp = this->last; 
				this->last = Previous(this->last); 
				this->last->next = NULL; 
				delete temp; 
			}
			else { 
				temp = Previous(it); 
				temp->next = it->next; 
				delete it; 
			}
			LeaveCriticalSection(&cs);
			return true;
		}
		it = forward;
	}
	LeaveCriticalSection(&cs);
	return false;
}

BOOL chatlist::SaveConfig()
{
	EnterCriticalSection(&cs);

	DWORD NumWritten = 0;

	std::stringstream filename;
	filename << dir << "\\Plugins\\" << "winamp.dat";


	HANDLE hFile;

	if((hFile = CreateFile(filename.str().c_str(), GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, 
		FILE_ATTRIBUTE_NORMAL, NULL)) == INVALID_HANDLE_VALUE){ LeaveCriticalSection(&cs); return false; }


	listitem *pos = this->first;
	listitem *forward;
	while(pos != 0)
	{
		WriteFile(hFile, pos->str, sizeof(pos->str), &NumWritten, NULL);
		pos = pos->next;
	}
	LeaveCriticalSection(&cs);
	CloseHandle(hFile);
	return true;
}

BOOL chatlist::LoadConfig()
{
	EnterCriticalSection(&cs);
	std::stringstream folder;

	std::stringstream filename;
	filename << dir << "\\Plugins\\" << "winamp.dat";

	//LONG offSet = 0;
	DWORD bytesRead = 1;
	DWORD newpos = 0;
	DWORD getl = 0;
	BOOL chk = true;

	HANDLE hFile;

	if((hFile = CreateFile(filename.str().c_str(), GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 
		FILE_ATTRIBUTE_NORMAL, NULL)) == INVALID_HANDLE_VALUE) { LeaveCriticalSection(&cs); return false; }

	char str[400];
	while(chk == true && bytesRead != 0)
	{
		chk = ReadFile(hFile, &str, 400, &bytesRead, NULL);
		//offSet += MAX_READ;
		this->Add(str);
	}
	this->Remove(str);
	CloseHandle(hFile);

	LeaveCriticalSection(&cs);
	return true;
}